/*----------------------------------------------------------------------- 
 * 
 * File Name: ApplySearch.c
 *
 * Author: Brown, D. A.
 * 
 * Revision: $Id: ApplySearch.c,v 1.27 2002/06/25 16:28:17 duncan Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include "ApplySearch.h"

NRCSID( APPLYSEARCHC, PKG "$Id: ApplySearch.c,v 1.27 2002/06/25 16:28:17 duncan Exp $" );


void
LALApplySearch(
    LALStatus             *status,
    LALSearchOutput       *output,
    LALSearchInput        *input,
    LALApplySearchParams  *applyParams
              )
{
  InspiralSearchParams         *params          = NULL;
  FindChirpMasterParams        *masterParamsPtr = NULL;
  FindChirpSlaveParams         *slaveParamsPtr  = NULL;
  InspiralEvent                *eventList       = NULL;


  INITSTATUS( status, "LALApplySearch", APPLYSEARCHC );
  ATTATCHSTATUSPTR( status );


 /*
  *
  * check input, output and parameters
  *
  */


  ASSERT( output, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
  ASSERT( !(output->result), status, 
      APPLYSEARCHH_ENNUL, APPLYSEARCHH_MSGENNUL );

  ASSERT( applyParams, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
  ASSERT( applyParams->mpiParams, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
  ASSERT( applyParams->searchParams, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );

  /* pointers to parameter structures */
  params = (InspiralSearchParams *) applyParams->searchParams;
  masterParamsPtr = &(params->masterParams);
  slaveParamsPtr  = &(params->slaveParams);


 /*
  *
  * set up or update the mpi communications within the search comm
  *
  */


  /* if search master, get the initial number of nodes in the serach comm */
  if ( params->searchMaster )
  {
    /* only do this the first time the master is called */
    if ( ! masterParamsPtr->numSlaves )
    {
      /* allocate memory and check for success */
      masterParamsPtr->numSlaves = (UINT4 *) LALCalloc( 1, sizeof(UINT4) );
      if( ! masterParamsPtr->numSlaves )
      {
        ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
      }

      /* get the number of search slaves */
      MPI_Comm_size( *(applyParams->mpiParams->comm), 
          masterParamsPtr->numSlaves );

      /* subrtact the master to get the number of slaves */
      params->info.numNodes = *(masterParamsPtr->numSlaves) -= 1;

      if ( params->inspiralDebugFlag == fcBankMinimalMatch )
      {
        /* create the bank sim vector if it is required (master is 0) */
        LALU4CreateVector( status->statusPtr, &(masterParamsPtr->bankSentVec),
            *(masterParamsPtr->numSlaves) + 1 );
        CHECKSTATUSPTR( status );

        memset( masterParamsPtr->bankSentVec->data, 0, 
            masterParamsPtr->bankSentVec->length  * sizeof(UINT4) );

        /* set the number of templates to filter: faked for progress */
        masterParamsPtr->numTmpltsToFilter = 
          *(masterParamsPtr->numSlaves) * (params->simCount + 1);
      }
    }
  }


 /*
  *
  * set up the internal parameters structures
  *
  */


  if ( params->searchMaster )
  {
    masterParamsPtr->mpiComm = applyParams->mpiParams->comm;
  }
  else /* search slave */
  {
    slaveParamsPtr->mpiComm = applyParams->mpiParams->comm;
  }


  /*
   *
   * execute the appropriate master or slave engine
   *
   */


  /* set the recieved event list and output to NULL */
  eventList             = NULL;
  output->numOutput     = 0;
  output->result        = NULL;

  if ( params->searchMaster )
  {
    /* call the inspiral master engine: this manages the distribution   */
    /* of templates and progess information. input and output are       */
    /* handled by the slaves unless we are doing a bank simulation      */
    LALFindChirpMaster( status->statusPtr, &eventList, masterParamsPtr );
    CHECKSTATUSPTR( status );

    /* compute progress information for wrapperAPI */
    if ( *(masterParamsPtr->numSlaves) )
    {
      output->fracRemaining = 
        ( (REAL4) masterParamsPtr->numTmpltsToFilter - 
          (REAL4) masterParamsPtr->numTmpltsFiltered ) / 
        (REAL4) masterParamsPtr->numTmpltsToFilter;
      output->notFinished   = 1;
    }
    else
    {
      output->fracRemaining = 0;
      output->notFinished   = 0;
    }
  }
  else /* search slave */
  {
    /* set the pointer to the wrapperAPIs not finished flag     */
    slaveParamsPtr->notFinished = &(output->notFinished);

    /* call the inspiral slave engine: this will return with    */
    /* output->notFinished set appropriately and possible a     */
    /* a list of inspiral events                                */
    LALFindChirpSlave( status->statusPtr, &eventList, 
        params->dataSegVec, slaveParamsPtr );
    CHECKSTATUSPTR( status );

    /* since we are operating in searchMaster/searchSlave mode  */
    /* fracRemaining is ignored on the search slaves. just set  */
    /* it to a sensible value                                   */
    output->fracRemaining = 0;

  } /* end if master/slave */


 /*
  *
  * if events are returned, generate the outPut structure 
  *
  */


  if ( eventList )
  {
    BuildInspiralOutPut( status->statusPtr, output, eventList );
    CHECKSTATUSPTR( status );

    while ( eventList )
    {
      InspiralEvent *thisEvent = eventList;
      eventList = eventList->next;
      LALFree (thisEvent);
      thisEvent = NULL;
    }
  }

  /* auxillary outputs (i.e. non inspiral events ) */
  if ( ! params->searchMaster )
  {
    /* if the filter output was requested and this is not the last loop */
    /* through the slave, append the snrsq vec to the output strcuture  */
    if ( params->slaveParams.filterParams->rhosqVec && output->notFinished )
    {
      BuildFilterOutPut( status->statusPtr, output, 
          params->slaveParams.filterParams->rhosqVec, 
          params->dataSegVec->data->chan->deltaT );
      CHECKSTATUSPTR( status );
    }

    /* create the standard candle output if it exists, */
    /* but make sure we only do this once              */
    if ( params->slaveParams.candlePtr )
    {
      BuildCandleOutPut( status->statusPtr, output, 
          params->slaveParams.candlePtr, &(params->info) );
        CHECKSTATUSPTR( status );

      LALFree( params->slaveParams.candlePtr );
      params->slaveParams.candlePtr = NULL;
    }
  }
  else if ( ! output->notFinished )
  {
    /* if this is a real search then create an entry for the */
    /* search summary table now that the job has finished    */
    dataBase   *dbase = NULL;
    LALBLOB     blob;
    UINT4       thisOutput = output->numOutput++;

    output->result = (outPut *) 
      LALRealloc( output->result, output->numOutput * sizeof( outPut ) );
    if ( ! output->result )
    {
      ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
    }

    memset( output->result + thisOutput, 0, sizeof( outPut ) );
    
    output->result[thisOutput].search = binaryInspiral;
    output->result[thisOutput].significant = 1;

    params->info.dsoName = "ldasinspiral";

    /* column "comment" */
    LALCreateNextDBEntry( status->statusPtr, &dbase, 
        "search_summary", "comment" );
    CHECKSTATUSPTR( status );
    blob.data = params->chan.name;
    blob.size = strlen( params->chan.name );
    LALCreateDBEntryBLOB( status->statusPtr, dbase, &blob, 1 );
    CHECKSTATUSPTR( status );
    output->result[thisOutput].results = dbase;

    LALCreateSearchSummaryDataBase( status->statusPtr, 
        &(dbase->next), &(params->info) );
    CHECKSTATUSPTR( status );
  }


  /* normal exit */
  DETATCHSTATUSPTR (status);
  RETURN (status);
}
