/********************************** <lalVerbatim file="TFCInitSearchCV">
Author: Julien Sylvestre
$Id: InitSearch.c,v 1.12 2002/04/16 19:18:01 julien Exp $
**************************************************** </lalVerbatim> */

#include <lal/LALStdlib.h>
#include <lal/TFClusters.h>
#include <lal/AVFactories.h>
#include <InitSearch.h>
#include <unistd.h>

NRCSID( INITSEARCHC, "tfclusters $Id: InitSearch.c,v 1.12 2002/04/16 19:18:01 julien Exp $");


/* PARAMETERS:
   argv[0] = "-filterparams"
   [1] = channel name; if SNGL_BURST, first two letters go into IFO. If GDS_TRIGGER, full name goes into subtype.
   [2] = table (SNGL_BURST or GDS_TRIGGER)
   [3] = total number of data
   [4] = number of samples in one segment
   [5] = time resolution in seconds
   [6] = sampling frequency (Hz)
   [7] = overlap between segments (number of samples)
   [8] = black pixel probability
   [9] = absolute error goal on power threshld. Set to <0 for white noise of unit variance at input. Set to 0 exactly and pass a spectrum to bypass Rice estimation. If a spectrum is passed, it must be from DC to Nyquist inclusively, and have a resolution 1/argv[5]. It should use a rectangular window, or else bpp will be different from argv[8]. A spectrum must be passed if a transfer function is passed.
   [10] = alpha
   [11] = minimum frequency to process (Hz)
   [12] = maximum frequency to process (Hz)
   [13] = number of standard deviations above estimated mean to reject in second pass at mean and variance estimation
   [14] = sigma
   [15] = distance for s1 = 1, s2 = 1
   [16] = distance for s1 = 1, s2 = 2
   ...
   [.] = distance for s1 = 1, s2 = sigma - 1
   [..] = distance for s1 = 2, s2 = 2
   ...
   [15+sigma*(sigma-1)/2] = distance for s1 = sigma-1, s2 = sigma-2
   [16+sigma*(sigma-1)/2] : if argument is present, writes pixel coordinates to database table sngl_transdata

*/

void LALInitSearch(
    LALStatus             *status,
    void                 **searchParams,
    LALInitSearchParams   *initSearchParams
    )
{

  TFCSearchParams *params;
  REAL8 T, alpha=0, minf, maxf;
  INT4 sigma, tmpi4;
  UINT4 i, olap, delL;
  INT4 fs, Ttotal, NData;
  UINT4 *delta;
  REAL4Vector *vect;
  REAL4 bpp, eGoal, Nsigma;
  char *name;
  DbTable table;
  BOOLEAN wBlobs = 0;

  INITSTATUS (status, "LALInitSearch", INITSEARCHC);
  ATTATCHSTATUSPTR (status);


  /*
   *
   * Sanity checks on the input, output, params
   *
   */

  /*
  printf("Sleeping\n");
  usleep(10000000);
  printf("Waking up\n");
  */


  if ( !(initSearchParams) ){
    ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL);
  }
  if ( (*searchParams) ){
    ABORT( status, INITSEARCHH_ENNUL, INITSEARCHH_MSGENNUL);
  }

  if ( !(initSearchParams->argv) ){
    ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL);
  }

  if ( initSearchParams->argc < 15){
        ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS ); 
  }
 
  if (strcmp( initSearchParams->argv[0], "-filterparams" )){
    ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS);
  }

  name = initSearchParams->argv[1];
  if(!strlen(name)) {
    ABORT ( status, INITSEARCHH_ENAM, INITSEARCHH_MSGENAM );
  }

  if(!strcmp(initSearchParams->argv[2], "SNGL_BURST")) {
    table = SNGL_BURST;
  }
  else {
    if(!strcmp(initSearchParams->argv[2], "GDS_TRIGGER")) {
      table = GDS_TRIGGER;
    }
    else {
      ABORT ( status, INITSEARCHH_ETAB, INITSEARCHH_MSGETAB );
    }
  }

  NData = atoi(initSearchParams->argv[3]);
  if ( NData <= 0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }


  Ttotal = atoi(initSearchParams->argv[4]);
  if ( Ttotal <= 0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }

  T = atof(initSearchParams->argv[5]);
  if ( isnan(T) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( T <= 0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }

  fs = atoi(initSearchParams->argv[6]);
  if ( fs <= 0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }


  olap = (unsigned int)atoi(initSearchParams->argv[7]);
  if ( olap < 0 ) {
    ABORT ( status, INITSEARCHH_EPOS, INITSEARCHH_MSGEPOS );
  }

  bpp = atof(initSearchParams->argv[8]);
  if ( isnan(bpp) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( bpp < 0.0 || bpp > 1.0 ) {
    ABORT ( status, INITSEARCHH_EAOR, INITSEARCHH_MSGEAOR );
  }

  eGoal = atof(initSearchParams->argv[9]);
  if ( isnan(eGoal) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }

  alpha = atof(initSearchParams->argv[10]);
  if ( isnan(alpha) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  /*
  if ( alpha < 0.0 || alpha > 1.0 ) {
    ABORT ( status, INITSEARCHH_EAOR, INITSEARCHH_MSGEAOR );
  }
  */

  minf = atof(initSearchParams->argv[11]);
  if ( isnan(minf) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( minf < 0.0 ) {
    ABORT ( status, INITSEARCHH_EPOS, INITSEARCHH_MSGEPOS );
  }

  maxf = atof(initSearchParams->argv[12]);
  if ( isnan(maxf) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( maxf < 0.0 ) {
    ABORT ( status, INITSEARCHH_EPOS, INITSEARCHH_MSGEPOS );
  }

  if( minf > maxf ) {
    ABORT ( status, INITSEARCHH_EMMF, INITSEARCHH_MSGEMMF );
  }

  Nsigma = atof(initSearchParams->argv[13]);
  if(Nsigma <= 0) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }
  
  sigma = atoi(initSearchParams->argv[14]);
  if(sigma<=0) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }

  delL = sigma * (sigma - 1) / 2;

  if(initSearchParams->argc != 15 + delL) {
    if(initSearchParams->argc == 16 + delL) {
      wBlobs = 1;
    } else {
      ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS );
    }
  }

  if(sigma > 1) {

    delta = (UINT4 *) LALMalloc ( delL * sizeof(UINT4) );
    if(!delta) {
      ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
    }

    for(i=0; i<delL; i++) {
    
      tmpi4 = atoi(initSearchParams->argv[15+i]);
      if(tmpi4 < 0) {
	ABORT ( status, INITSEARCHH_EPOS, INITSEARCHH_MSGEPOS );
      }

      delta[i] = (UINT4)tmpi4;
    }
  }
  else delta = NULL;


  /*
   *
   * allocate memory 
   *
   */

  *searchParams = params = LALMalloc (sizeof( TFCSearchParams )); 
  if ( !params )
  {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }


  params->dir = (CListDir*)LALMalloc(sizeof(CListDir));
  if(!params->dir) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }

  params->tseries = (REAL4TimeSeries*) LALMalloc (sizeof(REAL4TimeSeries));
  if(!params->tseries) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }

  params->tspec = (TFPlaneParams*) LALMalloc (sizeof(TFPlaneParams));
  if(!params->tspec) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }


  vect = NULL;

  params->tseries->deltaT=1.0/(REAL8)fs;
  params->tseries->f0 = 0.0;
  params->tseries->data = vect;

  params->dir->freqBins = 1 + (UINT4)(fs*T/2.0);
  params->dir->sigma = (UINT4)sigma;
  params->dir->minf = minf;
  params->dir->maxf = maxf;
  params->dir->alpha = alpha;
  params->Nsigma = Nsigma;

  LALFillCListDir(status->statusPtr, params->dir, 1E30); /* note: allocates memory for params->dir->rho from DC to Nyquist, and set to 1E30 */
  CHECKSTATUSPTR (status);

  if(delta) {
    memcpy(params->dir->d, delta, delL * sizeof(UINT4));
  
    params->dir->mdist = delta[0];
    for(i=1; i<delL; i++)
      if(delta[i] > params->dir->mdist) {params->dir->mdist = delta[i];}

    LALFree(delta);
  }
  else params->dir->mdist = 0;

  /*
   * 
   * assign the initial parameter values
   *
   */
  
  params->fs = fs;
  params->NData = (UINT4)NData;
  params->T = T;
  params->Ttotal = (UINT4)Ttotal;
  params->olap = olap;
  params->bpp = bpp;
  params->eGoal = eGoal;

  params->searchMaster = initSearchParams->nodeClass;

  params->dmro = NULL;

  memcpy(params->name, name, strlen(name)+1);

  params->table = table;

  params->resp = params->spec = NULL;

  params->WriteBlobs = wBlobs;
  
  DETATCHSTATUSPTR (status);
  RETURN (status);
}

